@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2018 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -13,8 +13,10 @@ from lib389.cli_base import (
     _generic_list,
     _generic_get,
     _get_arg,
+    CustomHelpFormatter
     )
 from lib389.cli_conf.monitor import _format_status
+from lib389.utils import get_passwd_from_file
 
 arg_to_attr = {
         'conn_bind_limit': 'nsbindconnectionslimit',
@@ -158,6 +160,10 @@ def def_config_set(inst, basedn, log, args):
 
 
 def create_link(inst, basedn, log, args, warn=True):
+    if args.bind_pw_file is not None:
+        args.bind_pw = get_passwd_from_file(args.bind_pw_file)
+    elif args.bind_pw_prompt:
+        args.bind_pw = _get_arg(None, msg="Enter remote bind DN password", hidden=True, confirm=True)
     attrs = _args_to_attrs(args)
     attrs['cn'] = args.CHAIN_NAME[0]
     links = ChainingLinks(inst)
@@ -172,6 +178,10 @@ def get_link(inst, basedn, log, args, warn=True):
 
 def edit_link(inst, basedn, log, args):
     chain_link = _get_link(inst, args.CHAIN_NAME[0])
+    if args.bind_pw_file is not None:
+        args.bind_pw = get_passwd_from_file(args.bind_pw_file)
+    elif args.bind_pw_prompt:
+        args.bind_pw = _get_arg(None, msg="Enter remote bind DN password", hidden=True, confirm=True)
     attrs = _args_to_attrs(args)
     did_something = False
     replace_list = []
@@ -211,80 +221,105 @@ def list_links(inst, basedn, log, args):
 
 
 def create_parser(subparsers):
-    chaining_parser = subparsers.add_parser('chaining', help="Manage database chaining/database links")
+    chaining_parser = subparsers.add_parser('chaining', help="Manage database chaining and database links", formatter_class=CustomHelpFormatter)
     subcommands = chaining_parser.add_subparsers(help="action")
 
-    config_get_parser = subcommands.add_parser('config-get', help='Get the chaining controls and server component lists')
+    config_get_parser = subcommands.add_parser('config-get', help='Display the chaining controls and server component lists', formatter_class=CustomHelpFormatter)
     config_get_parser.set_defaults(func=config_get)
-    config_get_parser.add_argument('--avail-controls', action='store_true', help="List available controls for chaining")
-    config_get_parser.add_argument('--avail-comps', action='store_true', help="List available plugin components for chaining")
+    config_get_parser.add_argument('--avail-controls', action='store_true', help="Lists available chaining controls")
+    config_get_parser.add_argument('--avail-comps', action='store_true', help="Lists available chaining plugin components")
 
-    config_set_parser = subcommands.add_parser('config-set', help='Set the chaining controls and server component lists')
+    config_set_parser = subcommands.add_parser('config-set', help='Set the chaining controls and server component lists', formatter_class=CustomHelpFormatter)
     config_set_parser.set_defaults(func=config_set)
-    config_set_parser.add_argument('--add-control', action='append', help="Add a transmitted control OID")
-    config_set_parser.add_argument('--del-control', action='append', help="Delete a transmitted control OID")
-    config_set_parser.add_argument('--add-comp', action='append', help="Add a chaining component")
-    config_set_parser.add_argument('--del-comp', action='append', help="Delete a chaining component")
+    config_set_parser.add_argument('--add-control', action='append', help="Adds a transmitted control OID")
+    config_set_parser.add_argument('--del-control', action='append', help="Deletes a transmitted control OID")
+    config_set_parser.add_argument('--add-comp', action='append', help="Adds a chaining component")
+    config_set_parser.add_argument('--del-comp', action='append', help="Deletes a chaining component")
 
-    def_config_get_parser = subcommands.add_parser('config-get-def', help='Get the default creation parameters for new database links')
+    def_config_get_parser = subcommands.add_parser('config-get-def', help='Display the default creation parameters for new database links', formatter_class=CustomHelpFormatter)
     def_config_get_parser.set_defaults(func=def_config_get)
 
-    def_config_set_parser = subcommands.add_parser('config-set-def', help='Set the default creation parameters for new database links')
+    def_config_set_parser = subcommands.add_parser('config-set-def', help='Set the default creation parameters for new database links', formatter_class=CustomHelpFormatter)
     def_config_set_parser.set_defaults(func=def_config_set)
-    def_config_set_parser.add_argument('--conn-bind-limit', help="The maximum number of BIND connections the database link establishes with the remote server.")
-    def_config_set_parser.add_argument('--conn-op-limit', help="The maximum number of LDAP connections the database link establishes with the remote server. ")
-    def_config_set_parser.add_argument('--abandon-check-interval', help="The number of seconds that pass before the server checks for abandoned operations.")
-    def_config_set_parser.add_argument('--bind-limit', help="The maximum number of concurrent bind operations per TCP connection.")
-    def_config_set_parser.add_argument('--op-limit', help="The maximum number of concurrent operations allowed.")
+    def_config_set_parser.add_argument('--conn-bind-limit',
+                                       help="Sets the maximum number of BIND connections the database link establishes "
+                                            "with the remote server")
+    def_config_set_parser.add_argument('--conn-op-limit',
+                                       help="Sets the maximum number of LDAP connections the database link establishes "
+                                            "with the remote server ")
+    def_config_set_parser.add_argument('--abandon-check-interval',
+                                       help="Sets the number of seconds that pass before the server checks for "
+                                            "abandoned operations")
+    def_config_set_parser.add_argument('--bind-limit',
+                                       help="Sets the maximum number of concurrent bind operations per TCP connection")
+    def_config_set_parser.add_argument('--op-limit', help="Sets the maximum number of concurrent operations allowed")
     def_config_set_parser.add_argument('--proxied-auth',
-        help="Set to \"off\" to disable proxied authorization, then binds for chained operations are executed as the user set in the nsMultiplexorBindDn attribute (on/off).")
-    def_config_set_parser.add_argument('--conn-lifetime', help="Specifies connection lifetime in seconds.  0 keeps connection open forever.")
-    def_config_set_parser.add_argument('--bind-timeout', help="The amount of time in seconds before a bind attempt times out.")
-    def_config_set_parser.add_argument('--return-ref', help="Sets whether referrals are returned by scoped searches (on/off).")
-    def_config_set_parser.add_argument('--check-aci', help="Set whether ACIs are evaluated on the database link as well as the remote data server (on/off).")
-    def_config_set_parser.add_argument('--bind-attempts', help="Sets the number of times the server tries to bind with the remote server.")
-    def_config_set_parser.add_argument('--size-limit', help="Sets the maximum number of entries to return from a search operation.")
-    def_config_set_parser.add_argument('--time-limit', help="Sets the maximum number of seconds allowed for an operation.")
+                                       help="Enables or disables proxied authorization. If set to \"off\", the server "
+                                            "executes bind for chained operations as the user set in the "
+                                            "nsMultiplexorBindDn attribute.")
+    def_config_set_parser.add_argument('--conn-lifetime',
+                                       help="Specifies connection lifetime in seconds. \"0\" keeps the connection open forever.")
+    def_config_set_parser.add_argument('--bind-timeout', help="Sets the amount of time in seconds before a bind attempt times out")
+    def_config_set_parser.add_argument('--return-ref', help="Enables or disables whether referrals are returned by scoped searches")
+    def_config_set_parser.add_argument('--check-aci',
+                                       help="Enables or disables whether the server evaluates ACIs on the database "
+                                            "link as well as the remote data server")
+    def_config_set_parser.add_argument('--bind-attempts', help="Sets the number of times the server tries to bind to the remote server")
+    def_config_set_parser.add_argument('--size-limit', help="Sets the maximum number of entries to return from a search operation")
+    def_config_set_parser.add_argument('--time-limit', help="Sets the maximum number of seconds allowed for an operation")
     def_config_set_parser.add_argument('--hop-limit',
-        help="Sets the maximum number of times a database is allowed to chain; that is, the number of times a request can be forwarded from one database link to another.")
+                                       help="Sets the maximum number of times a database is allowed to chain. That is "
+                                             "the number of times a request can be forwarded from one database link to another.")
     def_config_set_parser.add_argument('--response-delay',
-        help="The maximum amount of time it can take a remote server to respond to an LDAP operation request made by a database link before an error is suspected.")
-    def_config_set_parser.add_argument('--test-response-delay', help="Sets the duration of the test issued by the database link to check whether the remote server is responding.")
-    def_config_set_parser.add_argument('--use-starttls', help="Set to \"on\" specifies that the database links should use StartTLS for its secure connections.")
+                                       help="Sets the maximum amount of time it can take a remote server to respond to "
+                                            "an LDAP operation request made by a database link before an error is suspected")
+    def_config_set_parser.add_argument('--test-response-delay',
+                                       help="Sets the duration of the test issued by the database link to check whether "
+                                            "the remote server is responding")
+    def_config_set_parser.add_argument('--use-starttls', help="Configured that database links use StartTLS if set to \"on\"")
 
-    create_link_parser = subcommands.add_parser('link-create', add_help=False, conflict_handler='resolve', parents=[def_config_set_parser],
-        help='Create a database link to a remote server')
+    create_link_parser = subcommands.add_parser('link-create', add_help=False, conflict_handler='resolve',
+                                                parents=[def_config_set_parser],
+                                                help='Create a database link to a remote server')
     create_link_parser.set_defaults(func=create_link)
     create_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The name of the database link')
-    create_link_parser.add_argument('--suffix', required=True, help="The suffix managed by the database link.")
-    create_link_parser.add_argument('--server-url', required=True, help="Gives the LDAP/LDAPS URL of the remote server.")
+    create_link_parser.add_argument('--suffix', required=True, help="Sets the suffix managed by the database link")
+    create_link_parser.add_argument('--server-url', required=True, help="Sets the LDAP/LDAPS URL to the remote server")
     create_link_parser.add_argument('--bind-mech', required=True,
-        help="Sets the authentication method to use to authenticate to the remote server: SIMPLE, EXTERNAL, DIGEST-MD5, or GSSAPI. Default if unset is SIMPLE.")
-    create_link_parser.add_argument('--bind-dn', required=True, help="DN of the administrative entry used to communicate with the remote server")
-    create_link_parser.add_argument('--bind-pw', required=True, help="Password for the administrative user.")
+                                    help="Sets the authentication method to use to authenticate to the remote server. "
+                                         "Valid values: \"SIMPLE\" (default), \"EXTERNAL\", \"DIGEST-MD5\", or \"GSSAPI\"")
+    create_link_parser.add_argument('--bind-dn', required=True,
+                                    help="Sets the DN of the administrative entry used to communicate with the remote server")
+    create_link_parser.add_argument('--bind-pw', help="Sets the password of the administrative user")
+    create_link_parser.add_argument('--bind-pw-file', help="File containing the password")
+    create_link_parser.add_argument('--bind-pw-prompt', action='store_true', help="Prompt for password")
 
-    get_link_parser = subcommands.add_parser('link-get', help='get chaining database link')
+    get_link_parser = subcommands.add_parser('link-get', help='Displays chaining database links', formatter_class=CustomHelpFormatter)
     get_link_parser.set_defaults(func=get_link)
-    get_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The chaining link name, or suffix, to retrieve')
+    get_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The chaining link name or suffix to retrieve')
 
     edit_link_parser = subcommands.add_parser('link-set', add_help=False, conflict_handler='resolve',
         parents=[def_config_set_parser], help='Edit a database link to a remote server')
     edit_link_parser.set_defaults(func=edit_link)
     edit_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The name of the database link')
-    edit_link_parser.add_argument('--suffix', help="The suffix managed by the database link.")
-    edit_link_parser.add_argument('--server-url', help="Gives the LDAP/LDAPS URL of the remote server.")
+    edit_link_parser.add_argument('--suffix', help="Sets the suffix managed by the database link")
+    edit_link_parser.add_argument('--server-url', help="Sets the LDAP/LDAPS URL to the remote server")
     edit_link_parser.add_argument('--bind-mech',
-        help="Sets the authentication method to use to authenticate to the remote server: SIMPLE, EXTERNAL, DIGEST-MD5, or GSSAPI. Default if unset is SIMPLE.")
-    edit_link_parser.add_argument('--bind-dn', help="DN of the administrative entry used to communicate with the remote server")
-    edit_link_parser.add_argument('--bind-pw', help="Password for the administrative user.")
+                                  help="Sets the authentication method to use to authenticate to the remote server: "
+                                       "Valid values: \"SIMPLE\" (default), \"EXTERNAL\", \"DIGEST-MD5\", or \"GSSAPI\"")
+    edit_link_parser.add_argument('--bind-dn',
+                                  help="Sets the DN of the administrative entry used to communicate with the remote server")
+    edit_link_parser.add_argument('--bind-pw', help="Sets the password of the administrative user")
+    edit_link_parser.add_argument('--bind-pw-file', help="File containing the password")
+    edit_link_parser.add_argument('--bind-pw-prompt', action='store_true', help="Prompt for password")
 
-    delete_link_parser = subcommands.add_parser('link-delete', help='Delete a database link')
+    delete_link_parser = subcommands.add_parser('link-delete', help='Delete a database link', formatter_class=CustomHelpFormatter)
     delete_link_parser.set_defaults(func=delete_link)
     delete_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The name of the database link')
 
-    monitor_link_parser = subcommands.add_parser('monitor', help='Get the monitor information for a database chaining link')
+    monitor_link_parser = subcommands.add_parser('monitor', help='Display monitor information for a database chaining link', formatter_class=CustomHelpFormatter)
     monitor_link_parser.set_defaults(func=monitor_link)
     monitor_link_parser.add_argument('CHAIN_NAME', nargs=1, help='The name of the database link')
 
-    list_link_parser = subcommands.add_parser('link-list', help='List database links')
+    list_link_parser = subcommands.add_parser('link-list', help='List database links', formatter_class=CustomHelpFormatter)
     list_link_parser.set_defaults(func=list_links)
