@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -16,22 +16,15 @@ import socket
 import time
 import shutil
 import logging
-# from nss import nss
 import subprocess
+import uuid
 from datetime import datetime, timedelta
 from subprocess import check_output, run, PIPE
 from lib389.passwd import password_generate
 from lib389._mapped_object_lint import DSLint
 from lib389.lint import DSCERTLE0001, DSCERTLE0002
-from lib389.utils import ensure_str, format_cmd_list
-import uuid
+from lib389.utils import ensure_str, format_cmd_list, DSVersion, cert_is_ca
 
-# Setuptools ships with 'packaging' module, let's use it from there
-try:
-    from pkg_resources.extern.packaging.version import LegacyVersion
-# Fallback to a normal 'packaging' module in case 'setuptools' is stripped
-except:
-    from packaging.version import LegacyVersion
 
 KEYBITS = 4096
 CA_NAME = 'Self-Signed-CA'
@@ -51,7 +44,7 @@ log = logging.getLogger(__name__)
 
 
 class NssSsl(DSLint):
-    def __init__(self, dirsrv=None, dbpassword=None, dbpath=None):
+    def __init__(self, dirsrv=None, dbpassword=None, dbpath=None, uid=None, gid=None):
         self.dirsrv = dirsrv
         self._certdb = dbpath
         if self._certdb is None:
@@ -132,7 +125,6 @@ class NssSsl(DSLint):
         :type alt_names: [str, ]
         :returns: String of the subject DN.
         """
-
         if self.dirsrv and len(alt_names) > 0:
             return SELF_ISSUER.format(GIVENNAME=self.dirsrv.get_uuid(), HOSTNAME=alt_names[0])
         elif len(alt_names) > 0:
@@ -158,7 +150,7 @@ class NssSsl(DSLint):
         finally:
             prv_mask = os.umask(prv_mask)
 
-    def reinit(self):
+    def reinit(self, uid=None, gid=None):
         """
         Re-init (create) the nss db.
         """
@@ -198,6 +190,10 @@ only.
             if not os.path.exists(pwd_text_file):
                 with open(pwd_text_file, 'w') as f:
                     f.write('%s' % self.dbpassword)
+
+            if uid is not None and gid is not None:
+                os.chown(pin_file, uid, gid)
+                os.chown(pwd_text_file, uid, gid)
         finally:
             prv_mask = os.umask(prv_mask)
 
@@ -211,8 +207,14 @@ only.
         except subprocess.CalledProcessError as e:
             raise ValueError(e.output.decode('utf-8').rstrip())
         self.log.debug("nss output: %s", result)
+
+        if uid is not None and gid is not None:
+            for file in self.db_files["sql_backend"]:
+                os.chown(file, uid, gid)
+
         return True
 
+
     def _db_exists(self, even_partial=False):
         """Check that a nss db exists at the certpath"""
 
@@ -250,7 +252,7 @@ only.
             openssl_version = check_output(['/usr/bin/openssl', 'version']).decode('utf-8').strip()
         except subprocess.CalledProcessError as e:
             raise ValueError(e.output.decode('utf-8').rstrip())
-        rehash_available = LegacyVersion(openssl_version.split(' ')[1]) >= LegacyVersion('1.1.0')
+        rehash_available = DSVersion(openssl_version.split(' ')[1]) >= DSVersion('1.1.0')
 
         if rehash_available:
             cmd = ['/usr/bin/openssl', 'rehash', certdir]
@@ -461,6 +463,126 @@ only.
             cert_values.append(re.match(r'^(.+[^\s])[\s]+([^\s]+)$', line.rstrip()).groups())
         return cert_values
 
+    def _openssl_get_csr_subject(self, csr_dir, csr_name):
+        cmd = [
+            '/usr/bin/openssl',
+            'req',
+            '-subject',
+            '-noout',
+            '-in',
+            '%s/%s'% (csr_dir, csr_name),
+        ]
+        self.log.debug("cmd: %s", format_cmd_list(cmd))
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        # Parse the subject string from openssl output
+        result = result.replace("subject=", "")
+        result = result.replace(" ", "").strip()
+        result = result.split(',')
+        result = result[slice(None, None, -1)]
+        result = ','.join([str(elem) for elem in result])
+        return result
+
+    def _openssl_get_csr_sub_alt_names(self, csr_dir, csr_name):
+        cmd = [
+            '/usr/bin/openssl',
+            'req',
+            '-noout',
+            '-text',
+            '-in',
+            '%s/%s'% (csr_dir, csr_name),
+        ]
+        self.log.debug("cmd: %s", format_cmd_list(cmd))
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        subaltnames = []
+        lines = result.split('\n')
+        for line in lines:
+            if 'DNS:' in line:
+                names = line.split(',')
+                for altname in names:
+                    altname = altname.replace("DNS:", "")
+                    subaltnames.append(altname.replace(",:", "").strip())
+
+        return subaltnames
+
+    def _csr_show(self, name):
+        csr_dir = self.dirsrv.get_cert_dir()
+        result = ""
+        # Display PEM contents of a CSR file
+        if name and csr_dir:
+            if os.path.exists(csr_dir + "/" + name + ".csr"):
+                cmd = [
+                    "/usr/bin/sed",
+                    "-n",
+                    '/BEGIN NEW/,/END NEW/p',
+                    csr_dir + "/" + name + ".csr"
+                ]
+                self.log.debug("cmd: %s", format_cmd_list(cmd))
+                try:
+                    result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+                except subprocess.CalledProcessError as e:
+                    raise ValueError(e.output.decode('utf-8').rstrip())
+
+        return result
+
+    def _csr_list(self, csr_dir=None):
+        csr_list = []
+        csr_dir = self.dirsrv.get_cert_dir()
+        # Search for .csr file extensions in instance config dir
+        cmd = [
+            '/usr/bin/find',
+            csr_dir,
+            '-type',
+            'f',
+            '-name',
+            '*.csr',
+            '-printf',
+            '%f\\n',
+        ]
+        self.log.debug("cmd: %s", format_cmd_list(cmd))
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        # Bail out if we cant find any .csr files
+        if len(result) == 0:
+            return []
+
+        # For each .csr file, get last modified time and subject DN
+        for csr_file in result.splitlines():
+            csr = []
+            # Get last modified time stamp
+            cmd = [
+                '/usr/bin/date',
+                '-r',
+                '%s/%s'% (csr_dir, csr_file),
+                '+%Y-%m-%d %H:%M:%S',
+            ]
+            try:
+                result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+            except subprocess.CalledProcessError as e:
+                raise ValueError(e.output.decode('utf-8').rstrip())
+
+            # Add csr modified timestamp
+            csr.append(result.strip())
+            # Use openssl to get the csr subject DN
+            csr.append(self._openssl_get_csr_subject(csr_dir, csr_file))
+            # Use openssl to get the csr subject alt host names
+            csr.append(self._openssl_get_csr_sub_alt_names(csr_dir, csr_file))
+            # Add csr name, without extension
+            csr.append(csr_file.rsplit('.', 1)[0])
+            csr_list.append(csr)
+
+        return csr_list
+
     def _rsa_cert_key_exists(self, cert_tuple):
         name = cert_tuple[0]
         cmd = [
@@ -478,7 +600,7 @@ only.
             raise ValueError(e.output.decode('utf-8').rstrip())
         lines = result.split('\n')[1:-1]
         for line in lines:
-            m = re.match('\<(?P<id>.*)\> (?P<type>\w+)\s+(?P<hash>\w+).*:(?P<name>.+)', line)
+            m = re.match(r'\<(?P<id>.*)\> (?P<type>\w+)\s+(?P<hash>\w+).*:(?P<name>.+)', line)
             if name == m.group('name'):
                 return True
         return False
@@ -587,18 +709,22 @@ only.
         self.log.debug("nss output: %s", result)
         return True
 
-    def create_rsa_key_and_csr(self, alt_names=[], subject=None):
+    def create_rsa_key_and_csr(self, alt_names=[], subject=None, name=None):
         """Create a new RSA key and the certificate signing request. This
         request can be submitted to a CA for signing. The returned certificate
         can be added with import_rsa_crt.
         """
-        csr_path = os.path.join(self._certdb, '%s.csr' % CERT_NAME)
+        if name is None:
+            csr_path = os.path.join(self._certdb, '%s.csr' % CERT_NAME)
+        else:
+            csr_path = os.path.join(self._certdb, '%s.csr' % name)
 
         if len(alt_names) == 0:
             alt_names = self.detect_alt_names(alt_names)
         if subject is None:
             subject = self.generate_cert_subject(alt_names)
 
+        self.log.debug(f"CSR name -> {name}")
         self.log.debug(f"CSR subject -> {subject}")
         self.log.debug(f"CSR alt_names -> {alt_names}")
 
@@ -856,6 +982,52 @@ only.
         crt_der_path = '%s/%s%s.der' % (self._certdb, USER_PREFIX, name)
         return {'ca': ca_path, 'key': key_path, 'crt': crt_path, 'crt_der_path': crt_der_path}
 
+    def list_keys(self, orphan=None):
+        key_list = []
+        cmd = [
+            '/usr/bin/certutil',
+            '-K',
+            '-d',
+            self._certdb,
+            '-f',
+            '%s/%s' % (self._certdb, PWD_TXT),
+        ]
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        # Ignore the first line of certutil output
+        for line in result.splitlines()[1:]:
+            # Normalise the output of certutil
+            line = re.sub(r"\<[^>]*\>","", line)
+            key = re.split(r'\s{2,}', line)
+            if orphan:
+                if 'orphan' in line:
+                    key_list.append(key)
+            else:
+                key_list.append(key)
+
+        return key_list
+
+    def del_key(self, keyid):
+        cmd = [
+            '/usr/bin/certutil',
+            '-F',
+            '-d',
+            self._certdb,
+            '-f',
+            '%s/%s' % (self._certdb, PWD_TXT),
+            '-k',
+            keyid,
+        ]
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        return result
+
     # Certificate helper functions
     def del_cert(self,  nickname):
         """Delete this certificate
@@ -907,7 +1079,6 @@ only.
         except subprocess.CalledProcessError as e:
             raise ValueError(e.output.decode('utf-8').rstrip())
 
-
     def display_cert_details(self, nickname):
         cmd = [
             '/usr/bin/certutil',
@@ -925,7 +1096,6 @@ only.
 
         return ensure_str(result)
 
-
     def get_cert_details(self, nickname):
         """Get the trust flags, subject DN, issuer, and expiration date
 
@@ -1000,7 +1170,6 @@ only.
             if self._rsa_cert_is_caclienttrust(cert)
         ]
 
-
     def add_cert(self, nickname, input_file, ca=False):
         """Add server or CA cert
         """
@@ -1009,11 +1178,21 @@ only.
         if not os.path.exists(input_file):
             raise ValueError("The certificate file ({}) does not exist".format(input_file))
 
-        self._assert_not_chain(input_file)
+        pem_file = True
+        if not input_file.lower().endswith(".pem"):
+            pem_file = False
+        else:
+            self._assert_not_chain(input_file)
 
         if ca:
+            # Verify this is a CA cert
+            if not cert_is_ca(input_file):
+                raise ValueError(f"Certificate ({nickname}) is not a CA certificate")
             trust_flags = "CT,,"
         else:
+            # Verify this is a server cert
+            if cert_is_ca(input_file):
+                raise ValueError(f"Certificate ({nickname}) is not a server certificate")
             trust_flags = ",,"
 
         cmd = [
@@ -1023,10 +1202,12 @@ only.
             '-n', nickname,
             '-t', trust_flags,
             '-i', input_file,
-            '-a',
             '-f',
             '%s/%s' % (self._certdb, PWD_TXT),
         ]
+        if pem_file:
+            cmd.append('-a')
+
         self.log.debug("add_cert cmd: %s", format_cmd_list(cmd))
         try:
             check_output(cmd, stderr=subprocess.STDOUT)
@@ -1088,3 +1269,145 @@ only.
             # Remove the p12
             if os.path.exists(p12_bundle):
                 os.remove(p12_bundle)
+
+    def add_ca_cert_bundle(self, cert_file, nicknames):
+        """
+        Add a PEM file that could be a bundle of CA certs
+
+        :param nicknames: list of names of each CA certificate
+        :param cert_file: path to certificate PEM file
+        :raises:
+        """
+
+        # Verify input_file exists
+        if not os.path.exists(cert_file):
+            raise ValueError("The certificate file ({}) does not exist".format(cert_file))
+
+        if not cert_file.lower().endswith(".pem"):
+            # Binary cert, this can not be a bundle
+            self.add_cert(nicknames[0], cert_file, ca=True)
+            log.info(f"Successfully added CA certificate ({nicknames[0]})")
+            return
+
+        try:
+            with open(cert_file, 'r') as f:
+                ca_count = 0
+                ca_files_to_cleanup = []
+                writing_ca = False
+                try:
+                    for line in f:
+                        if line.startswith('-----BEGIN CERTIFICATE-----'):
+                            # create tmp cert file
+                            writing_ca = True
+                            ca_file_name = cert_file + "-" + str(ca_count)
+                            ca_file = open(ca_file_name, 'w')
+                            ca_files_to_cleanup.append(ca_file_name)
+                            ca_file.write(line)
+                        elif writing_ca:
+                            ca_file.write(line)
+                            if line.startswith('-----END CERTIFICATE-----'):
+                                writing_ca = False
+                                ca_file.close()
+
+                                # Generate CA certificate nickname
+                                names_len = len(nicknames)
+                                if names_len > ca_count:
+                                    if nicknames[ca_count].lower() == CERT_NAME.lower() or nicknames[ca_count].lower() == CA_NAME.lower():
+                                        # cleanup
+                                        try:
+                                            for tmp_file in ca_files_to_cleanup:
+                                                os.remove(tmp_file)
+                                        except IOError as e:
+                                            # failed to remove tmp cert
+                                            log.debug("Failed to remove tmp cert file: " + str(e))
+                                        raise ValueError(f"You may not import a CA with the nickname {CERT_NAME} or {CA_NAME}")
+                                    ca_cert_name = nicknames[ca_count]
+                                else:
+                                    # A name was not provided for this cert, create one
+                                    ca_cert_name = nicknames[names_len - 1] + str(ca_count)
+
+                                # Check if certificate nickname exists
+                                name_exists = False
+                                try:
+                                    self.get_cert_details(ca_cert_name)
+                                    name_exists = True
+                                except ValueError:
+                                    pass
+
+                                if name_exists:
+                                    # Not good, cleanup and raise error
+                                    try:
+                                        for tmp_file in ca_files_to_cleanup:
+                                            os.remove(tmp_file)
+                                    except IOError as e:
+                                        # failed to remove tmp cert
+                                        log.debug("Failed to remove tmp cert file: " + str(e))
+                                    raise ValueError(f"Certificate already exists with the same name ({ca_cert_name})")
+
+                                # Verify this is a CA cert
+                                if not cert_is_ca(ca_file_name):
+                                    raise ValueError(f"Certificate ({ca_cert_name}) is not a CA certificate")
+
+                                # Add this CA certificate
+                                self.add_cert(ca_cert_name, ca_file_name, ca=True)
+                                ca_count += 1
+                                log.info(f"Successfully added CA certificate ({ca_cert_name})")
+
+                    # All done, cleanup
+                    try:
+                        for tmp_file in ca_files_to_cleanup:
+                            os.remove(tmp_file)
+                    except IOError as e:
+                        # failed to remove tmp cert
+                        log.debug("Failed to remove tmp cert file: " + str(e))
+                except IOError as e:
+                    # Some failure, remove all the tmp files
+                    ca_file.close()
+                    try:
+                        for tmp_file in ca_files_to_cleanup:
+                            os.remove(tmp_file)
+                    except IOError as e:
+                        # failed to remove tmp cert
+                        log.debug("Failed to remove tmp cert file: " + str(e))
+                    raise e
+        except EnvironmentError as e:
+            raise e
+
+    def export_cert(self, nickname, output_file=None, der_format=False):
+        """
+        :param nickname: name of certificate
+        :param output_file: name for exported certificate
+        :param der_format: export certificate in DER/binary format
+        :raise ValueError: error
+        """
+        cmd = [
+            '/usr/bin/certutil',
+            '-L',
+            '-d', self._certdb,
+            '-n', nickname,
+            '-f',
+            '%s/%s' % (self._certdb, PWD_TXT),
+        ]
+
+        # Handle args for PEM vs DER options
+        if der_format:
+            cmd.append('-r')
+            if output_file is None:
+                output_file = nickname + ".crt"
+        else:
+            cmd.append('-a')
+            if output_file is None:
+                output_file = nickname + ".pem"
+
+        # Set output file name
+        if not output_file.startswith("/"):
+            # Must be full path, otherwise but it in the cert dir
+            output_file = f"{self._certdb}/{output_file}"
+        cmd.append('-o')
+        cmd.append(output_file)
+
+        self.log.debug("export_cert cmd: %s", format_cmd_list(cmd))
+        try:
+            check_output(cmd, stderr=subprocess.STDOUT)
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
